/*******************************************************************************
*   Author       -  Kiraly Tibor
*                   www.tkiraaly.hu
*   Date         -  2016.10.27.
*   Chip         -  Atmel AVR
*   Compiler     -  avr-gcc (WinAVR)
*
*   AVR macros
*
*******************************************************************************/

#ifndef tkiraaly_avr
#define tkiraaly_avr




#define _AVR_IO_H_                               // portok egyebek definialasa nem kell
#include <avr/interrupt.h>                       // ISR( vector) definicioja




#define U8                   unsigned char
#define U16                  unsigned int




#define KHZ                  *1000UL
#define MHZ                  *1000000UL




#define B_1(sfr,bit)         sfr|= ( 1 << ( bit))               // egy regiszter adott bitjenek 1-re allitas
#define B_0(sfr,bit)         sfr&= ~( 1 << ( bit))              // egy regiszter adott bitjenek 0-ra allitas
#define B_R(sfr,bit)         ( sfr & ( 1 << ( bit)))            // egy regiszter egy bitjenek olvasasa




#define NOP                  __asm__ __volatile__ ("nop" ::)    // egy ures utasitas beszurasa




#define IT_ENABLE            __asm__ __volatile__ ("sei" ::)    // megszakitas engedelyezese
#define IT_DISABLE           __asm__ __volatile__ ("cli" ::)    // megszakitas tiltasa




#define USART_UCSRC          0B10000000          // UCSRC kivalasztasa
#define USART_ASYNCRON       0                   // asszinkron mukodes     - UCSR_C
#define USART_SYNCRON        0B01000000          // szinkron mukodes       - UCSR_C
#define USART_RX_ENABLE      0B00010000          // vetel engedelyezese    - UCSR_B
#define USART_TX_ENABLE      0B00001000          // adas engedelyezese     - UCSR_B
#define USART_NO_PARITY      0                   // nincs paritas bit      - UCSR_C
#define USART_EVEN_PARITY    0B00100000          // paros paritas          - UCSR_C
#define USART_ODD_PARITY     0B00110000          // paratlan paritas       - UCSR_C
#define USART_1_STOPBIT      0                   // 1 stopbit              - UCSR_C
#define USART_2_STOPBITS     0B00001000          // 2 stopbit              - UCSR_C
#define USART_7_BIT          0B00000100          // 7 bit-es karakterhossz - UCSR_C
#define USART_8_BIT          0B00000110          // 8 bit-es karakterhossz - UCSR_C




#define OCR_TIMER( ido_usec, osztas)  ido_usec* FCPU/ osztas/ 1000000UL+ 1UL
#define OCR_CLOCK( f_hz, osztas)      F_CPU/ osztas/ f_hz- 1UL
#define OCR_WAVE( f_hz, osztas)       F_CPU/ 2UL/ osztas/ f_hz- 1UL




/*******************************************************************************
*
*   Port/pin kezelo objektumhoz
*   X_PORT     - port szama 
*   PXY_PIN    - pin szama
*   
*   Port/pin kezelo macro-k
*   PXY_OUTPUT - pin-t kimenetre allitasa
*   PXY_INPUT  - pin-t bemenetre allitasa
*   PXY_PULLUP - pin-t bemenetre allitasa es abelso felhuzast bekapcsolasa
*   PXY_1      - pin-t 1-be allitasa
*   PXY_0      - pin-t 0-ba allitasa
*   PXY_RD     - pin-t lekerdezo/kiolvasasa
*   PXY_WR()   - pin-t iro/beallitasa
*   
*     Y = 0..1 - adott bit
*     Y = A    - osszes bit
*     Y = U    - upper/felso nibble/fel byte
*     Y = L    - lower/also nibble/fel byte
*
*******************************************************************************/

#define A_PORT                0x0000
#define PA0_PIN               A_PORT+ 0B00000001
#define PA1_PIN               A_PORT+ 0B00000010
#define PA2_PIN               A_PORT+ 0B00000100
#define PA3_PIN               A_PORT+ 0B00001000
#define PA4_PIN               A_PORT+ 0B00010000
#define PA5_PIN               A_PORT+ 0B00100000
#define PA6_PIN               A_PORT+ 0B01000000
#define PA7_PIN               A_PORT+ 0B10000000

#define PA0_OUTPUT            B_1( DDRA, 0)
#define PA1_OUTPUT            B_1( DDRA, 1)
#define PA2_OUTPUT            B_1( DDRA, 2)
#define PA3_OUTPUT            B_1( DDRA, 3)
#define PA4_OUTPUT            B_1( DDRA, 4)
#define PA5_OUTPUT            B_1( DDRA, 5)
#define PA6_OUTPUT            B_1( DDRA, 6)
#define PA7_OUTPUT            B_1( DDRA, 7)
#define PAU_OUTPUT            DDRA|= 0B11110000
#define PAL_OUTPUT            DDRA|= 0B00001111
#define PAA_OUTPUT            DDRA=  0B11111111

#define PA0_INPUT             B_0( DDRA, 0)
#define PA1_INPUT             B_0( DDRA, 1)
#define PA2_INPUT             B_0( DDRA, 2)
#define PA3_INPUT             B_0( DDRA, 3)
#define PA4_INPUT             B_0( DDRA, 4)
#define PA5_INPUT             B_0( DDRA, 5)
#define PA6_INPUT             B_0( DDRA, 6)
#define PA7_INPUT             B_0( DDRA, 7)
#define PAU_INPUT             DDRA&= 0B00001111
#define PAL_INPUT             DDRA&= 0B11110000
#define PAA_INPUT             DDRA=  0

#define PA0_PULLUP            PA0_INPUT; PA0_1
#define PA1_PULLUP            PA1_INPUT; PA1_1
#define PA2_PULLUP            PA2_INPUT; PA2_1
#define PA3_PULLUP            PA3_INPUT; PA3_1
#define PA4_PULLUP            PA4_INPUT; PA4_1
#define PA5_PULLUP            PA5_INPUT; PA5_1
#define PA6_PULLUP            PA6_INPUT; PA6_1
#define PA7_PULLUP            PA7_INPUT; PA7_1
#define PAU_PULLUP            PAU_INPUT; PAU_1
#define PAL_PULLUP            PAL_INPUT; PAL_1
#define PAA_PULLUP            PAA_INPUT; PAA_1

#define PA0_1                 B_1( PORTA, 0)
#define PA1_1                 B_1( PORTA, 1)
#define PA2_1                 B_1( PORTA, 2)
#define PA3_1                 B_1( PORTA, 3)
#define PA4_1                 B_1( PORTA, 4)
#define PA5_1                 B_1( PORTA, 5)
#define PA6_1                 B_1( PORTA, 6)
#define PA7_1                 B_1( PORTA, 7)
#define PAU_1                 PORTA|= 0B11110000
#define PAL_1                 PORTA|= 0B00001111
#define PAA_1                 PORTA=  0B11111111

#define PA0_0                 B_0( PORTA, 0)
#define PA1_0                 B_0( PORTA, 1)
#define PA2_0                 B_0( PORTA, 2)
#define PA3_0                 B_0( PORTA, 3)
#define PA4_0                 B_0( PORTA, 4)
#define PA5_0                 B_0( PORTA, 5)
#define PA6_0                 B_0( PORTA, 6)
#define PA7_0                 B_0( PORTA, 7)
#define PAU_0                 PORTA&= 0B00001111
#define PAL_0                 PORTA&= 0B11110000
#define PAA_0                 PORTA=  0B00000000

#define PA0_RD                B_R( PINA, 0)
#define PA1_RD                B_R( PINA, 1)
#define PA2_RD                B_R( PINA, 2)
#define PA3_RD                B_R( PINA, 3)
#define PA4_RD                B_R( PINA, 4)
#define PA5_RD                B_R( PINA, 5)
#define PA6_RD                B_R( PINA, 6)
#define PA7_RD                B_R( PINA, 7)
#define PAU_RD                (( PINA & 0B11110000) >> 4)
#define PAL_RD                (PINA & 0B00001111)
#define PAA_RD                PINA

#define PA0_WR( X)            X ? PA0_1 : PA0_0
#define PA1_WR( X)            X ? PA1_1 : PA1_0
#define PA2_WR( X)            X ? PA2_1 : PA2_0
#define PA3_WR( X)            X ? PA3_1 : PA3_0
#define PA4_WR( X)            X ? PA4_1 : PA4_0
#define PA5_WR( X)            X ? PA5_1 : PA5_0
#define PA6_WR( X)            X ? PA6_1 : PA6_0
#define PA7_WR( X)            X ? PA7_1 : PA7_0
#define PAU_WR( X)            ( PORTA& 0B00001111)| ( X<< 4)
#define PAL_WR( X)            ( PORTA& 0B11110000)| ( X & 0B00001111)
#define PAA_WR( X)            PORTA= X




#define B_PORT                0x0100
#define PB0_PIN               B_PORT+ 0B00000001
#define PB1_PIN               B_PORT+ 0B00000010
#define PB2_PIN               B_PORT+ 0B00000100
#define PB3_PIN               B_PORT+ 0B00001000
#define PB4_PIN               B_PORT+ 0B00010000
#define PB5_PIN               B_PORT+ 0B00100000
#define PB6_PIN               B_PORT+ 0B01000000
#define PB7_PIN               B_PORT+ 0B10000000

#define PB0_OUTPUT            B_1( DDRB, 0)
#define PB1_OUTPUT            B_1( DDRB, 1)
#define PB2_OUTPUT            B_1( DDRB, 2)
#define PB3_OUTPUT            B_1( DDRB, 3)
#define PB4_OUTPUT            B_1( DDRB, 4)
#define PB5_OUTPUT            B_1( DDRB, 5)
#define PB6_OUTPUT            B_1( DDRB, 6)
#define PB7_OUTPUT            B_1( DDRB, 7)
#define PBU_OUTPUT            DDRB|= 0B11110000
#define PBL_OUTPUT            DDRB|= 0B00001111
#define PBA_OUTPUT            DDRB=  0B11111111

#define PB0_INPUT             B_0( DDRB, 0)
#define PB1_INPUT             B_0( DDRB, 1)
#define PB2_INPUT             B_0( DDRB, 2)
#define PB3_INPUT             B_0( DDRB, 3)
#define PB4_INPUT             B_0( DDRB, 4)
#define PB5_INPUT             B_0( DDRB, 5)
#define PB6_INPUT             B_0( DDRB, 6)
#define PB7_INPUT             B_0( DDRB, 7)
#define PBU_INPUT             DDRB&= 0B00001111
#define PBL_INPUT             DDRB&= 0B11110000
#define PBA_INPUT             DDRB=  0

#define PB0_PULLUP            PB0_INPUT; PB0_1
#define PB1_PULLUP            PB1_INPUT; PB1_1
#define PB2_PULLUP            PB2_INPUT; PB2_1
#define PB3_PULLUP            PB3_INPUT; PB3_1
#define PB4_PULLUP            PB4_INPUT; PB4_1
#define PB5_PULLUP            PB5_INPUT; PB5_1
#define PB6_PULLUP            PB6_INPUT; PB6_1
#define PB7_PULLUP            PB7_INPUT; PB7_1
#define PBU_PULLUP            PBU_INPUT; PBU_1
#define PBL_PULLUP            PBL_INPUT; PBL_1
#define PBA_PULLUP            PBA_INPUT; PBA_1

#define PB0_1                 B_1( PORTB, 0)
#define PB1_1                 B_1( PORTB, 1)
#define PB2_1                 B_1( PORTB, 2)
#define PB3_1                 B_1( PORTB, 3)
#define PB4_1                 B_1( PORTB, 4)
#define PB5_1                 B_1( PORTB, 5)
#define PB6_1                 B_1( PORTB, 6)
#define PB7_1                 B_1( PORTB, 7)
#define PBU_1                 PORTB|= 0B11110000
#define PBL_1                 PORTB|= 0B00001111
#define PBA_1                 PORTB=  0B11111111

#define PB0_0                 B_0( PORTB, 0)
#define PB1_0                 B_0( PORTB, 1)
#define PB2_0                 B_0( PORTB, 2)
#define PB3_0                 B_0( PORTB, 3)
#define PB4_0                 B_0( PORTB, 4)
#define PB5_0                 B_0( PORTB, 5)
#define PB6_0                 B_0( PORTB, 6)
#define PB7_0                 B_0( PORTB, 7)
#define PBU_0                 PORTB&= 0B00001111
#define PBL_0                 PORTB&= 0B11110000
#define PBA_0                 PORTB=  0B00000000

#define PB0_RD                B_R( PINB, 0)
#define PB1_RD                B_R( PINB, 1)
#define PB2_RD                B_R( PINB, 2)
#define PB3_RD                B_R( PINB, 3)
#define PB4_RD                B_R( PINB, 4)
#define PB5_RD                B_R( PINB, 5)
#define PB6_RD                B_R( PINB, 6)
#define PB7_RD                B_R( PINB, 7)
#define PBU_RD                (( PINB & 0B11110000) >> 4)
#define PBL_RD                (PINB & 0B00001111)
#define PBA_RD                PINB

#define PB0_WR( X)            X ? PB0_1 : PB0_0
#define PB1_WR( X)            X ? PB1_1 : PB1_0
#define PB2_WR( X)            X ? PB2_1 : PB2_0
#define PB3_WR( X)            X ? PB3_1 : PB3_0
#define PB4_WR( X)            X ? PB4_1 : PB4_0
#define PB5_WR( X)            X ? PB5_1 : PB5_0
#define PB6_WR( X)            X ? PB6_1 : PB6_0
#define PB7_WR( X)            X ? PB7_1 : PB7_0
#define PBU_WR( X)            ( PORTB& 0B00001111)| ( X<< 4)
#define PBL_WR( X)            ( PORTB& 0B11110000)| ( X & 0B00001111)
#define PBA_WR( X)            PORTB= X




#define C_PORT                0x0200
#define PC0_PIN               C_PORT+ 0B00000001
#define PC1_PIN               C_PORT+ 0B00000010
#define PC2_PIN               C_PORT+ 0B00000100
#define PC3_PIN               C_PORT+ 0B00001000
#define PC4_PIN               C_PORT+ 0B00010000
#define PC5_PIN               C_PORT+ 0B00100000
#define PC6_PIN               C_PORT+ 0B01000000
#define PC7_PIN               C_PORT+ 0B10000000

#define PC0_OUTPUT            B_1( DDRC, 0)
#define PC1_OUTPUT            B_1( DDRC, 1)
#define PC2_OUTPUT            B_1( DDRC, 2)
#define PC3_OUTPUT            B_1( DDRC, 3)
#define PC4_OUTPUT            B_1( DDRC, 4)
#define PC5_OUTPUT            B_1( DDRC, 5)
#define PC6_OUTPUT            B_1( DDRC, 6)
#define PC7_OUTPUT            B_1( DDRC, 7)
#define PCU_OUTPUT            DDRC|= 0B11110000
#define PCL_OUTPUT            DDRC|= 0B00001111
#define PCA_OUTPUT            DDRC=  0B11111111

#define PC0_INPUT             B_0( DDRC, 0)
#define PC1_INPUT             B_0( DDRC, 1)
#define PC2_INPUT             B_0( DDRC, 2)
#define PC3_INPUT             B_0( DDRC, 3)
#define PC4_INPUT             B_0( DDRC, 4)
#define PC5_INPUT             B_0( DDRC, 5)
#define PC6_INPUT             B_0( DDRC, 6)
#define PC7_INPUT             B_0( DDRC, 7)
#define PCU_INPUT             DDRC&= 0B00001111
#define PCL_INPUT             DDRC&= 0B11110000
#define PCA_INPUT             DDRC=  0

#define PC0_PULLUP            PC0_INPUT; PC0_1
#define PC1_PULLUP            PC1_INPUT; PC1_1
#define PC2_PULLUP            PC2_INPUT; PC2_1
#define PC3_PULLUP            PC3_INPUT; PC3_1
#define PC4_PULLUP            PC4_INPUT; PC4_1
#define PC5_PULLUP            PC5_INPUT; PC5_1
#define PC6_PULLUP            PC6_INPUT; PC6_1
#define PC7_PULLUP            PC7_INPUT; PC7_1
#define PCU_PULLUP            PCU_INPUT; PCU_1
#define PCL_PULLUP            PCL_INPUT; PCL_1
#define PCA_PULLUP            PCA_INPUT; PCA_1

#define PC0_1                 B_1( PORTC, 0)
#define PC1_1                 B_1( PORTC, 1)
#define PC2_1                 B_1( PORTC, 2)
#define PC3_1                 B_1( PORTC, 3)
#define PC4_1                 B_1( PORTC, 4)
#define PC5_1                 B_1( PORTC, 5)
#define PC6_1                 B_1( PORTC, 6)
#define PC7_1                 B_1( PORTC, 7)
#define PCU_1                 PORTC|= 0B11110000
#define PCL_1                 PORTC|= 0B00001111
#define PCA_1                 PORTC=  0B11111111

#define PC0_0                 B_0( PORTC, 0)
#define PC1_0                 B_0( PORTC, 1)
#define PC2_0                 B_0( PORTC, 2)
#define PC3_0                 B_0( PORTC, 3)
#define PC4_0                 B_0( PORTC, 4)
#define PC5_0                 B_0( PORTC, 5)
#define PC6_0                 B_0( PORTC, 6)
#define PC7_0                 B_0( PORTC, 7)
#define PCU_0                 PORTC&= 0B00001111
#define PCL_0                 PORTC&= 0B11110000
#define PCA_0                 PORTC=  0B00000000

#define PC0_RD                B_R( PINC, 0)
#define PC1_RD                B_R( PINC, 1)
#define PC2_RD                B_R( PINC, 2)
#define PC3_RD                B_R( PINC, 3)
#define PC4_RD                B_R( PINC, 4)
#define PC5_RD                B_R( PINC, 5)
#define PC6_RD                B_R( PINC, 6)
#define PC7_RD                B_R( PINC, 7)
#define PCU_RD                (( PINC & 0B11110000) >> 4)
#define PCL_RD                (PINC & 0B00001111)
#define PCA_RD                PINC

#define PC0_WR( X)            X ? PC0_1 : PC0_0
#define PC1_WR( X)            X ? PC1_1 : PC1_0
#define PC2_WR( X)            X ? PC2_1 : PC2_0
#define PC3_WR( X)            X ? PC3_1 : PC3_0
#define PC4_WR( X)            X ? PC4_1 : PC4_0
#define PC5_WR( X)            X ? PC5_1 : PC5_0
#define PC6_WR( X)            X ? PC6_1 : PC6_0
#define PC7_WR( X)            X ? PC7_1 : PC7_0
#define PCU_WR( X)            ( PORTC& 0B00001111)| ( X<< 4)
#define PCL_WR( X)            ( PORTC& 0B11110000)| ( X & 0B00001111)
#define PCA_WR( X)            PORTC= X




#define D_PORT               0x0300
#define PD0_PIN               D_PORT+ 0B00000001
#define PD1_PIN               D_PORT+ 0B00000010
#define PD2_PIN               D_PORT+ 0B00000100
#define PD3_PIN               D_PORT+ 0B00001000
#define PD4_PIN               D_PORT+ 0B00010000
#define PD5_PIN               D_PORT+ 0B00100000
#define PD6_PIN               D_PORT+ 0B01000000
#define PD7_PIN               D_PORT+ 0B10000000

#define PD0_OUTPUT            B_1( DDRD, 0)
#define PD1_OUTPUT            B_1( DDRD, 1)
#define PD2_OUTPUT            B_1( DDRD, 2)
#define PD3_OUTPUT            B_1( DDRD, 3)
#define PD4_OUTPUT            B_1( DDRD, 4)
#define PD5_OUTPUT            B_1( DDRD, 5)
#define PD6_OUTPUT            B_1( DDRD, 6)
#define PD7_OUTPUT            B_1( DDRD, 7)
#define PDU_OUTPUT            DDRD|= 0B11110000
#define PDL_OUTPUT            DDRD|= 0B00001111
#define PDA_OUTPUT            DDRD=  0B11111111

#define PD0_INPUT             B_0( DDRD, 0)
#define PD1_INPUT             B_0( DDRD, 1)
#define PD2_INPUT             B_0( DDRD, 2)
#define PD3_INPUT             B_0( DDRD, 3)
#define PD4_INPUT             B_0( DDRD, 4)
#define PD5_INPUT             B_0( DDRD, 5)
#define PD6_INPUT             B_0( DDRD, 6)
#define PD7_INPUT             B_0( DDRD, 7)
#define PDU_INPUT             DDRD&= 0B00001111
#define PDL_INPUT             DDRD&= 0B11110000
#define PDA_INPUT             DDRD=  0

#define PD0_PULLUP            PD0_INPUT; PD0_1
#define PD1_PULLUP            PD1_INPUT; PD1_1
#define PD2_PULLUP            PD2_INPUT; PD2_1
#define PD3_PULLUP            PD3_INPUT; PD3_1
#define PD4_PULLUP            PD4_INPUT; PD4_1
#define PD5_PULLUP            PD5_INPUT; PD5_1
#define PD6_PULLUP            PD6_INPUT; PD6_1
#define PD7_PULLUP            PD7_INPUT; PD7_1
#define PDU_PULLUP            PDU_INPUT; PDU_1
#define PDL_PULLUP            PDL_INPUT; PDL_1
#define PDA_PULLUP            PDA_INPUT; PDA_1

#define PD0_1                 B_1( PORTD, 0)
#define PD1_1                 B_1( PORTD, 1)
#define PD2_1                 B_1( PORTD, 2)
#define PD3_1                 B_1( PORTD, 3)
#define PD4_1                 B_1( PORTD, 4)
#define PD5_1                 B_1( PORTD, 5)
#define PD6_1                 B_1( PORTD, 6)
#define PD7_1                 B_1( PORTD, 7)
#define PDU_1                 PORTD|= 0B11110000
#define PDL_1                 PORTD|= 0B00001111
#define PDA_1                 PORTD=  0B11111111

#define PD0_0                 B_0( PORTD, 0)
#define PD1_0                 B_0( PORTD, 1)
#define PD2_0                 B_0( PORTD, 2)
#define PD3_0                 B_0( PORTD, 3)
#define PD4_0                 B_0( PORTD, 4)
#define PD5_0                 B_0( PORTD, 5)
#define PD6_0                 B_0( PORTD, 6)
#define PD7_0                 B_0( PORTD, 7)
#define PDU_0                 PORTD&= 0B00001111
#define PDL_0                 PORTD&= 0B11110000
#define PDA_0                 PORTD=  0B00000000

#define PD0_RD                B_R( PIND, 0)
#define PD1_RD                B_R( PIND, 1)
#define PD2_RD                B_R( PIND, 2)
#define PD3_RD                B_R( PIND, 3)
#define PD4_RD                B_R( PIND, 4)
#define PD5_RD                B_R( PIND, 5)
#define PD6_RD                B_R( PIND, 6)
#define PD7_RD                B_R( PIND, 7)
#define PDU_RD                (( PIND & 0B11110000) >> 4)
#define PDL_RD                (PIND & 0B00001111)
#define PDA_RD                PIND

#define PD0_WR( X)            X ? PD0_1 : PD0_0
#define PD1_WR( X)            X ? PD1_1 : PD1_0
#define PD2_WR( X)            X ? PD2_1 : PD2_0
#define PD3_WR( X)            X ? PD3_1 : PD3_0
#define PD4_WR( X)            X ? PD4_1 : PD4_0
#define PD5_WR( X)            X ? PD5_1 : PD5_0
#define PD6_WR( X)            X ? PD6_1 : PD6_0
#define PD7_WR( X)            X ? PD7_1 : PD7_0
#define PDU_WR( X)            ( PORTD& 0B00001111)| ( X<< 4)
#define PDL_WR( X)            ( PORTD& 0B11110000)| ( X & 0B00001111)
#define PDA_WR( X)            PORTD= X




#define E_PORT               0x0400
#define PE0_PIN               E_PORT+ 0B00000001
#define PE1_PIN               E_PORT+ 0B00000010
#define PE2_PIN               E_PORT+ 0B00000100
#define PE3_PIN               E_PORT+ 0B00001000
#define PE4_PIN               E_PORT+ 0B00010000
#define PE5_PIN               E_PORT+ 0B00100000
#define PE6_PIN               E_PORT+ 0B01000000
#define PE7_PIN               E_PORT+ 0B10000000

#define PE0_OUTPUT            B_1( DDRE, 0)
#define PE1_OUTPUT            B_1( DDRE, 1)
#define PE2_OUTPUT            B_1( DDRE, 2)
#define PE3_OUTPUT            B_1( DDRE, 3)
#define PE4_OUTPUT            B_1( DDRE, 4)
#define PE5_OUTPUT            B_1( DDRE, 5)
#define PE6_OUTPUT            B_1( DDRE, 6)
#define PE7_OUTPUT            B_1( DDRE, 7)
#define PEU_OUTPUT            DDRE|= 0B11110000
#define PEL_OUTPUT            DDRE|= 0B00001111
#define PEA_OUTPUT            DDRE=  0B11111111

#define PE0_INPUT             B_0( DDRE, 0)
#define PE1_INPUT             B_0( DDRE, 1)
#define PE2_INPUT             B_0( DDRE, 2)
#define PE3_INPUT             B_0( DDRE, 3)
#define PE4_INPUT             B_0( DDRE, 4)
#define PE5_INPUT             B_0( DDRE, 5)
#define PE6_INPUT             B_0( DDRE, 6)
#define PE7_INPUT             B_0( DDRE, 7)
#define PEU_INPUT             DDRE&= 0B00001111
#define PEL_INPUT             DDRE&= 0B11110000
#define PEA_INPUT             DDRE=  0

#define PE0_PULLUP            PE0_INPUT; PE0_1
#define PE1_PULLUP            PE1_INPUT; PE1_1
#define PE2_PULLUP            PE2_INPUT; PE2_1
#define PE3_PULLUP            PE3_INPUT; PE3_1
#define PE4_PULLUP            PE4_INPUT; PE4_1
#define PE5_PULLUP            PE5_INPUT; PE5_1
#define PE6_PULLUP            PE6_INPUT; PE6_1
#define PE7_PULLUP            PE7_INPUT; PE7_1
#define PEU_PULLUP            PEU_INPUT; PEU_1
#define PEL_PULLUP            PEL_INPUT; PEL_1
#define PEA_PULLUP            PEA_INPUT; PEA_1

#define PE0_1                 B_1( PORTE, 0)
#define PE1_1                 B_1( PORTE, 1)
#define PE2_1                 B_1( PORTE, 2)
#define PE3_1                 B_1( PORTE, 3)
#define PE4_1                 B_1( PORTE, 4)
#define PE5_1                 B_1( PORTE, 5)
#define PE6_1                 B_1( PORTE, 6)
#define PE7_1                 B_1( PORTE, 7)
#define PEU_1                 PORTE|= 0B11110000
#define PEL_1                 PORTE|= 0B00001111
#define PEA_1                 PORTE=  0B11111111

#define PE0_0                 B_0( PORTE, 0)
#define PE1_0                 B_0( PORTE, 1)
#define PE2_0                 B_0( PORTE, 2)
#define PE3_0                 B_0( PORTE, 3)
#define PE4_0                 B_0( PORTE, 4)
#define PE5_0                 B_0( PORTE, 5)
#define PE6_0                 B_0( PORTE, 6)
#define PE7_0                 B_0( PORTE, 7)
#define PEU_0                 PORTE&= 0B00001111
#define PEL_0                 PORTE&= 0B11110000
#define PEA_0                 PORTE=  0B00000000

#define PE0_RD                B_R( PINE, 0)
#define PE1_RD                B_R( PINE, 1)
#define PE2_RD                B_R( PINE, 2)
#define PE3_RD                B_R( PINE, 3)
#define PE4_RD                B_R( PINE, 4)
#define PE5_RD                B_R( PINE, 5)
#define PE6_RD                B_R( PINE, 6)
#define PE7_RD                B_R( PINE, 7)
#define PEU_RD                (( PINE & 0B11110000) >> 4)
#define PEL_RD                (PINE & 0B00001111)
#define PEA_RD                PINE

#define PE0_WR( X)            X ? PE0_1 : PE0_0
#define PE1_WR( X)            X ? PE1_1 : PE1_0
#define PE2_WR( X)            X ? PE2_1 : PE2_0
#define PE3_WR( X)            X ? PE3_1 : PE3_0
#define PE4_WR( X)            X ? PE4_1 : PE4_0
#define PE5_WR( X)            X ? PE5_1 : PE5_0
#define PE6_WR( X)            X ? PE6_1 : PE6_0
#define PE7_WR( X)            X ? PE7_1 : PE7_0
#define PEU_WR( X)            ( PORTE& 0B00001111)| ( X<< 4)
#define PEL_WR( X)            ( PORTE& 0B11110000)| ( X & 0B00001111)
#define PEA_WR( X)            PORTE= X




#define F_PORT               0x0500
#define PF0_PIN               F_PORT+ 0B00000001
#define PF1_PIN               F_PORT+ 0B00000010
#define PF2_PIN               F_PORT+ 0B00000100
#define PF3_PIN               F_PORT+ 0B00001000
#define PF4_PIN               F_PORT+ 0B00010000
#define PF5_PIN               F_PORT+ 0B00100000
#define PF6_PIN               F_PORT+ 0B01000000
#define PF7_PIN               F_PORT+ 0B10000000

#define PF0_OUTPUT            B_1( DDRF, 0)
#define PF1_OUTPUT            B_1( DDRF, 1)
#define PF2_OUTPUT            B_1( DDRF, 2)
#define PF3_OUTPUT            B_1( DDRF, 3)
#define PF4_OUTPUT            B_1( DDRF, 4)
#define PF5_OUTPUT            B_1( DDRF, 5)
#define PF6_OUTPUT            B_1( DDRF, 6)
#define PF7_OUTPUT            B_1( DDRF, 7)
#define PFU_OUTPUT            DDRF|= 0B11110000
#define PFL_OUTPUT            DDRF|= 0B00001111
#define PFA_OUTPUT            DDRF=  0B11111111

#define PF0_INPUT             B_0( DDRF, 0)
#define PF1_INPUT             B_0( DDRF, 1)
#define PF2_INPUT             B_0( DDRF, 2)
#define PF3_INPUT             B_0( DDRF, 3)
#define PF4_INPUT             B_0( DDRF, 4)
#define PF5_INPUT             B_0( DDRF, 5)
#define PF6_INPUT             B_0( DDRF, 6)
#define PF7_INPUT             B_0( DDRF, 7)
#define PFU_INPUT             DDRF&= 0B00001111
#define PFL_INPUT             DDRF&= 0B11110000
#define PFA_INPUT             DDRF=  0

#define PF0_PULLUP            PF0_INPUT; PF0_1
#define PF1_PULLUP            PF1_INPUT; PF1_1
#define PF2_PULLUP            PF2_INPUT; PF2_1
#define PF3_PULLUP            PF3_INPUT; PF3_1
#define PF4_PULLUP            PF4_INPUT; PF4_1
#define PF5_PULLUP            PF5_INPUT; PF5_1
#define PF6_PULLUP            PF6_INPUT; PF6_1
#define PF7_PULLUP            PF7_INPUT; PF7_1
#define PFU_PULLUP            PFU_INPUT; PFU_1
#define PFL_PULLUP            PFL_INPUT; PFL_1
#define PFA_PULLUP            PFA_INPUT; PFA_1

#define PF0_1                 B_1( PORTF, 0)
#define PF1_1                 B_1( PORTF, 1)
#define PF2_1                 B_1( PORTF, 2)
#define PF3_1                 B_1( PORTF, 3)
#define PF4_1                 B_1( PORTF, 4)
#define PF5_1                 B_1( PORTF, 5)
#define PF6_1                 B_1( PORTF, 6)
#define PF7_1                 B_1( PORTF, 7)
#define PFU_1                 PORTF|= 0B11110000
#define PFL_1                 PORTF|= 0B00001111
#define PFA_1                 PORTF=  0B11111111

#define PF0_0                 B_0( PORTF, 0)
#define PF1_0                 B_0( PORTF, 1)
#define PF2_0                 B_0( PORTF, 2)
#define PF3_0                 B_0( PORTF, 3)
#define PF4_0                 B_0( PORTF, 4)
#define PF5_0                 B_0( PORTF, 5)
#define PF6_0                 B_0( PORTF, 6)
#define PF7_0                 B_0( PORTF, 7)
#define PFU_0                 PORTF&= 0B00001111
#define PFL_0                 PORTF&= 0B11110000
#define PFA_0                 PORTF=  0B00000000

#define PF0_RD                B_R( PINF, 0)
#define PF1_RD                B_R( PINF, 1)
#define PF2_RD                B_R( PINF, 2)
#define PF3_RD                B_R( PINF, 3)
#define PF4_RD                B_R( PINF, 4)
#define PF5_RD                B_R( PINF, 5)
#define PF6_RD                B_R( PINF, 6)
#define PF7_RD                B_R( PINF, 7)
#define PFU_RD                (( PINF & 0B11110000) >> 4)
#define PFL_RD                (PINF & 0B00001111)
#define PFA_RD                PINF

#define PF0_WR( X)            X ? PF0_1 : PF0_0
#define PF1_WR( X)            X ? PF1_1 : PF1_0
#define PF2_WR( X)            X ? PF2_1 : PF2_0
#define PF3_WR( X)            X ? PF3_1 : PF3_0
#define PF4_WR( X)            X ? PF4_1 : PF4_0
#define PF5_WR( X)            X ? PF5_1 : PF5_0
#define PF6_WR( X)            X ? PF6_1 : PF6_0
#define PF7_WR( X)            X ? PF7_1 : PF7_0
#define PFU_WR( X)            ( PORTF& 0B00001111)| ( X<< 4)
#define PFL_WR( X)            ( PORTF& 0B11110000)| ( X & 0B00001111)
#define PFA_WR( X)            PORTF= X




#define G_PORT               0x0600
#define PG0_PIN               G_PORT+ 0B00000001
#define PG1_PIN               G_PORT+ 0B00000010
#define PG2_PIN               G_PORT+ 0B00000100
#define PG3_PIN               G_PORT+ 0B00001000
#define PG4_PIN               G_PORT+ 0B00010000
#define PG5_PIN               G_PORT+ 0B00100000
#define PG6_PIN               G_PORT+ 0B01000000
#define PG7_PIN               G_PORT+ 0B10000000

#define PG0_OUTPUT            B_1( DDRG, 0)
#define PG1_OUTPUT            B_1( DDRG, 1)
#define PG2_OUTPUT            B_1( DDRG, 2)
#define PG3_OUTPUT            B_1( DDRG, 3)
#define PG4_OUTPUT            B_1( DDRG, 4)
#define PG5_OUTPUT            B_1( DDRG, 5)
#define PG6_OUTPUT            B_1( DDRG, 6)
#define PG7_OUTPUT            B_1( DDRG, 7)
#define PGU_OUTPUT            DDRG|= 0B11110000
#define PGL_OUTPUT            DDRG|= 0B00001111
#define PGA_OUTPUT            DDRG=  0B11111111

#define PG0_INPUT             B_0( DDRG, 0)
#define PG1_INPUT             B_0( DDRG, 1)
#define PG2_INPUT             B_0( DDRG, 2)
#define PG3_INPUT             B_0( DDRG, 3)
#define PG4_INPUT             B_0( DDRG, 4)
#define PG5_INPUT             B_0( DDRG, 5)
#define PG6_INPUT             B_0( DDRG, 6)
#define PG7_INPUT             B_0( DDRG, 7)
#define PGU_INPUT             DDRG&= 0B00001111
#define PGL_INPUT             DDRG&= 0B11110000
#define PGA_INPUT             DDRG=  0

#define PG0_PULLUP            PG0_INPUT; PG0_1
#define PG1_PULLUP            PG1_INPUT; PG1_1
#define PG2_PULLUP            PG2_INPUT; PG2_1
#define PG3_PULLUP            PG3_INPUT; PG3_1
#define PG4_PULLUP            PG4_INPUT; PG4_1
#define PG5_PULLUP            PG5_INPUT; PG5_1
#define PG6_PULLUP            PG6_INPUT; PG6_1
#define PG7_PULLUP            PG7_INPUT; PG7_1
#define PGU_PULLUP            PGU_INPUT; PGU_1
#define PGL_PULLUP            PGL_INPUT; PGL_1
#define PGA_PULLUP            PGA_INPUT; PGA_1

#define PG0_1                 B_1( PORTG, 0)
#define PG1_1                 B_1( PORTG, 1)
#define PG2_1                 B_1( PORTG, 2)
#define PG3_1                 B_1( PORTG, 3)
#define PG4_1                 B_1( PORTG, 4)
#define PG5_1                 B_1( PORTG, 5)
#define PG6_1                 B_1( PORTG, 6)
#define PG7_1                 B_1( PORTG, 7)
#define PGU_1                 PORTG|= 0B11110000
#define PGL_1                 PORTG|= 0B00001111
#define PGA_1                 PORTG=  0B11111111

#define PG0_0                 B_0( PORTG, 0)
#define PG1_0                 B_0( PORTG, 1)
#define PG2_0                 B_0( PORTG, 2)
#define PG3_0                 B_0( PORTG, 3)
#define PG4_0                 B_0( PORTG, 4)
#define PG5_0                 B_0( PORTG, 5)
#define PG6_0                 B_0( PORTG, 6)
#define PG7_0                 B_0( PORTG, 7)
#define PGU_0                 PORTG&= 0B00001111
#define PGL_0                 PORTG&= 0B11110000
#define PGA_0                 PORTG=  0B00000000

#define PG0_RD                B_R( PING, 0)
#define PG1_RD                B_R( PING, 1)
#define PG2_RD                B_R( PING, 2)
#define PG3_RD                B_R( PING, 3)
#define PG4_RD                B_R( PING, 4)
#define PG5_RD                B_R( PING, 5)
#define PG6_RD                B_R( PING, 6)
#define PG7_RD                B_R( PING, 7)
#define PGU_RD                (( PING & 0B11110000) >> 4)
#define PGL_RD                (PING & 0B00001111)
#define PGA_RD                PING

#define PG0_WR( X)            X ? PG0_1 : PG0_0
#define PG1_WR( X)            X ? PG1_1 : PG1_0
#define PG2_WR( X)            X ? PG2_1 : PG2_0
#define PG3_WR( X)            X ? PG3_1 : PG3_0
#define PG4_WR( X)            X ? PG4_1 : PG4_0
#define PG5_WR( X)            X ? PG5_1 : PG5_0
#define PG6_WR( X)            X ? PG6_1 : PG6_0
#define PG7_WR( X)            X ? PG7_1 : PG7_0
#define PGU_WR( X)            ( PORTG& 0B00001111)| ( X<< 4)
#define PGL_WR( X)            ( PORTG& 0B11110000)| ( X & 0B00001111)
#define PGA_WR( X)            PORTG= X




#define H_PORT               0x0700
#define PH0_PIN               H_PORT+ 0B00000001
#define PH1_PIN               H_PORT+ 0B00000010
#define PH2_PIN               H_PORT+ 0B00000100
#define PH3_PIN               H_PORT+ 0B00001000
#define PH4_PIN               H_PORT+ 0B00010000
#define PH5_PIN               H_PORT+ 0B00100000
#define PH6_PIN               H_PORT+ 0B01000000
#define PH7_PIN               H_PORT+ 0B10000000

#define PH0_OUTPUT            B_1( DDRH, 0)
#define PH1_OUTPUT            B_1( DDRH, 1)
#define PH2_OUTPUT            B_1( DDRH, 2)
#define PH3_OUTPUT            B_1( DDRH, 3)
#define PH4_OUTPUT            B_1( DDRH, 4)
#define PH5_OUTPUT            B_1( DDRH, 5)
#define PH6_OUTPUT            B_1( DDRH, 6)
#define PH7_OUTPUT            B_1( DDRH, 7)
#define PHU_OUTPUT            DDRH|= 0B11110000
#define PHL_OUTPUT            DDRH|= 0B00001111
#define PHA_OUTPUT            DDRH=  0B11111111

#define PH0_INPUT             B_0( DDRH, 0)
#define PH1_INPUT             B_0( DDRH, 1)
#define PH2_INPUT             B_0( DDRH, 2)
#define PH3_INPUT             B_0( DDRH, 3)
#define PH4_INPUT             B_0( DDRH, 4)
#define PH5_INPUT             B_0( DDRH, 5)
#define PH6_INPUT             B_0( DDRH, 6)
#define PH7_INPUT             B_0( DDRH, 7)
#define PHU_INPUT             DDRH&= 0B00001111
#define PHL_INPUT             DDRH&= 0B11110000
#define PHA_INPUT             DDRH=  0

#define PH0_PULLUP            PH0_INPUT; PH0_1
#define PH1_PULLUP            PH1_INPUT; PH1_1
#define PH2_PULLUP            PH2_INPUT; PH2_1
#define PH3_PULLUP            PH3_INPUT; PH3_1
#define PH4_PULLUP            PH4_INPUT; PH4_1
#define PH5_PULLUP            PH5_INPUT; PH5_1
#define PH6_PULLUP            PH6_INPUT; PH6_1
#define PH7_PULLUP            PH7_INPUT; PH7_1
#define PHU_PULLUP            PHU_INPUT; PHU_1
#define PHL_PULLUP            PHL_INPUT; PHL_1
#define PHA_PULLUP            PHA_INPUT; PHA_1

#define PH0_1                 B_1( PORTH, 0)
#define PH1_1                 B_1( PORTH, 1)
#define PH2_1                 B_1( PORTH, 2)
#define PH3_1                 B_1( PORTH, 3)
#define PH4_1                 B_1( PORTH, 4)
#define PH5_1                 B_1( PORTH, 5)
#define PH6_1                 B_1( PORTH, 6)
#define PH7_1                 B_1( PORTH, 7)
#define PHU_1                 PORTH|= 0B11110000
#define PHL_1                 PORTH|= 0B00001111
#define PHA_1                 PORTH=  0B11111111

#define PH0_0                 B_0( PORTH, 0)
#define PH1_0                 B_0( PORTH, 1)
#define PH2_0                 B_0( PORTH, 2)
#define PH3_0                 B_0( PORTH, 3)
#define PH4_0                 B_0( PORTH, 4)
#define PH5_0                 B_0( PORTH, 5)
#define PH6_0                 B_0( PORTH, 6)
#define PH7_0                 B_0( PORTH, 7)
#define PHU_0                 PORTH&= 0B00001111
#define PHL_0                 PORTH&= 0B11110000
#define PHA_0                 PORTH=  0B00000000

#define PH0_RD                B_R( PINH, 0)
#define PH1_RD                B_R( PINH, 1)
#define PH2_RD                B_R( PINH, 2)
#define PH3_RD                B_R( PINH, 3)
#define PH4_RD                B_R( PINH, 4)
#define PH5_RD                B_R( PINH, 5)
#define PH6_RD                B_R( PINH, 6)
#define PH7_RD                B_R( PINH, 7)
#define PHU_RD                (( PINH & 0B11110000) >> 4)
#define PHL_RD                (PINH & 0B00001111)
#define PHA_RD                PINH

#define PH0_WR( X)            X ? PH0_1 : PH0_0
#define PH1_WR( X)            X ? PH1_1 : PH1_0
#define PH2_WR( X)            X ? PH2_1 : PH2_0
#define PH3_WR( X)            X ? PH3_1 : PH3_0
#define PH4_WR( X)            X ? PH4_1 : PH4_0
#define PH5_WR( X)            X ? PH5_1 : PH5_0
#define PH6_WR( X)            X ? PH6_1 : PH6_0
#define PH7_WR( X)            X ? PH7_1 : PH7_0
#define PHU_WR( X)            ( PORTH& 0B00001111)| ( X<< 4)
#define PHL_WR( X)            ( PORTH& 0B11110000)| ( X & 0B00001111)
#define PHA_WR( X)            PORTH= X




#define I_PORT               0x0800
#define PI0_PIN               Y_PORT+ 0B00000001
#define PI1_PIN               I_PORT+ 0B00000010
#define PI2_PIN               I_PORT+ 0B00000100
#define PI3_PIN               I_PORT+ 0B00001000
#define PI4_PIN               I_PORT+ 0B00010000
#define PI5_PIN               I_PORT+ 0B00100000
#define PI6_PIN               I_PORT+ 0B01000000
#define PI7_PIN               I_PORT+ 0B10000000

#define PI0_OUTPUT            B_1( DDRI, 0)
#define PI1_OUTPUT            B_1( DDRI, 1)
#define PI2_OUTPUT            B_1( DDRI, 2)
#define PI3_OUTPUT            B_1( DDRI, 3)
#define PI4_OUTPUT            B_1( DDRI, 4)
#define PI5_OUTPUT            B_1( DDRI, 5)
#define PI6_OUTPUT            B_1( DDRI, 6)
#define PI7_OUTPUT            B_1( DDRI, 7)
#define PIU_OUTPUT            DDRI|= 0B11110000
#define PIL_OUTPUT            DDRI|= 0B00001111
#define PIA_OUTPUT            DDRI=  0B11111111

#define PI0_INPUT             B_0( DDRI, 0)
#define PI1_INPUT             B_0( DDRI, 1)
#define PI2_INPUT             B_0( DDRI, 2)
#define PI3_INPUT             B_0( DDRI, 3)
#define PI4_INPUT             B_0( DDRI, 4)
#define PI5_INPUT             B_0( DDRI, 5)
#define PI6_INPUT             B_0( DDRI, 6)
#define PI7_INPUT             B_0( DDRI, 7)
#define PIU_INPUT             DDRI&= 0B00001111
#define PIL_INPUT             DDRI&= 0B11110000
#define PIA_INPUT             DDRI=  0

#define PI0_PULLUP            PI0_INPUT; PI0_1
#define PI1_PULLUP            PI1_INPUT; PI1_1
#define PI2_PULLUP            PI2_INPUT; PI2_1
#define PI3_PULLUP            PI3_INPUT; PI3_1
#define PI4_PULLUP            PI4_INPUT; PI4_1
#define PI5_PULLUP            PI5_INPUT; PI5_1
#define PI6_PULLUP            PI6_INPUT; PI6_1
#define PI7_PULLUP            PI7_INPUT; PI7_1
#define PIU_PULLUP            PIU_INPUT; PIU_1
#define PIL_PULLUP            PIL_INPUT; PIL_1
#define PIA_PULLUP            PIA_INPUT; PIA_1

#define PI0_1                 B_1( PORTI, 0)
#define PI1_1                 B_1( PORTI, 1)
#define PI2_1                 B_1( PORTI, 2)
#define PI3_1                 B_1( PORTI, 3)
#define PI4_1                 B_1( PORTI, 4)
#define PI5_1                 B_1( PORTI, 5)
#define PI6_1                 B_1( PORTI, 6)
#define PI7_1                 B_1( PORTI, 7)
#define PIU_1                 PORTI|= 0B11110000
#define PIL_1                 PORTI|= 0B00001111
#define PIA_1                 PORTI=  0B11111111

#define PI0_0                 B_0( PORTI, 0)
#define PI1_0                 B_0( PORTI, 1)
#define PI2_0                 B_0( PORTI, 2)
#define PI3_0                 B_0( PORTI, 3)
#define PI4_0                 B_0( PORTI, 4)
#define PI5_0                 B_0( PORTI, 5)
#define PI6_0                 B_0( PORTI, 6)
#define PI7_0                 B_0( PORTI, 7)
#define PIU_0                 PORTI&= 0B00001111
#define PIL_0                 PORTI&= 0B11110000
#define PIA_0                 PORTI=  0B00000000

#define PI0_RD                B_R( PINI, 0)
#define PI1_RD                B_R( PINI, 1)
#define PI2_RD                B_R( PINI, 2)
#define PI3_RD                B_R( PINI, 3)
#define PI4_RD                B_R( PINI, 4)
#define PI5_RD                B_R( PINI, 5)
#define PI6_RD                B_R( PINI, 6)
#define PI7_RD                B_R( PINI, 7)
#define PIU_RD                (( PINI & 0B11110000) >> 4)
#define PIL_RD                (PINI & 0B00001111)
#define PIA_RD                PINI

#define PI0_WR( X)            X ? PI0_1 : PI0_0
#define PI1_WR( X)            X ? PI1_1 : PI1_0
#define PI2_WR( X)            X ? PI2_1 : PI2_0
#define PI3_WR( X)            X ? PI3_1 : PI3_0
#define PI4_WR( X)            X ? PI4_1 : PI4_0
#define PI5_WR( X)            X ? PI5_1 : PI5_0
#define PI6_WR( X)            X ? PI6_1 : PI6_0
#define PI7_WR( X)            X ? PI7_1 : PI7_0
#define PIU_WR( X)            ( PORTI& 0B00001111)| ( X<< 4)
#define PIL_WR( X)            ( PORTI& 0B11110000)| ( X & 0B00001111)
#define PIA_WR( X)            PORTI= X




#define J_PORT               0x0900
#define PJ0_PIN               J_PORT+ 0B00000001
#define PJ1_PIN               J_PORT+ 0B00000010
#define PJ2_PIN               J_PORT+ 0B00000100
#define PJ3_PIN               J_PORT+ 0B00001000
#define PJ4_PIN               J_PORT+ 0B00010000
#define PJ5_PIN               J_PORT+ 0B00100000
#define PJ6_PIN               J_PORT+ 0B01000000
#define PJ7_PIN               J_PORT+ 0B10000000

#define PJ0_OUTPUT            B_1( DDRJ, 0)
#define PJ1_OUTPUT            B_1( DDRJ, 1)
#define PJ2_OUTPUT            B_1( DDRJ, 2)
#define PJ3_OUTPUT            B_1( DDRJ, 3)
#define PJ4_OUTPUT            B_1( DDRJ, 4)
#define PJ5_OUTPUT            B_1( DDRJ, 5)
#define PJ6_OUTPUT            B_1( DDRJ, 6)
#define PJ7_OUTPUT            B_1( DDRJ, 7)
#define PJU_OUTPUT            DDRJ|= 0B11110000
#define PJL_OUTPUT            DDRJ|= 0B00001111
#define PJA_OUTPUT            DDRJ=  0B11111111

#define PJ0_INPUT             B_0( DDRJ, 0)
#define PJ1_INPUT             B_0( DDRJ, 1)
#define PJ2_INPUT             B_0( DDRJ, 2)
#define PJ3_INPUT             B_0( DDRJ, 3)
#define PJ4_INPUT             B_0( DDRJ, 4)
#define PJ5_INPUT             B_0( DDRJ, 5)
#define PJ6_INPUT             B_0( DDRJ, 6)
#define PJ7_INPUT             B_0( DDRJ, 7)
#define PJU_INPUT             DDRJ&= 0B00001111
#define PJL_INPUT             DDRJ&= 0B11110000
#define PJA_INPUT             DDRJ=  0

#define PJ0_PULLUP            PJ0_INPUT; PJ0_1
#define PJ1_PULLUP            PJ1_INPUT; PJ1_1
#define PJ2_PULLUP            PJ2_INPUT; PJ2_1
#define PJ3_PULLUP            PJ3_INPUT; PJ3_1
#define PJ4_PULLUP            PJ4_INPUT; PJ4_1
#define PJ5_PULLUP            PJ5_INPUT; PJ5_1
#define PJ6_PULLUP            PJ6_INPUT; PJ6_1
#define PJ7_PULLUP            PJ7_INPUT; PJ7_1
#define PJU_PULLUP            PJU_INPUT; PJU_1
#define PJL_PULLUP            PJL_INPUT; PJL_1
#define PJA_PULLUP            PJA_INPUT; PJA_1

#define PJ0_1                 B_1( PORTJ, 0)
#define PJ1_1                 B_1( PORTJ, 1)
#define PJ2_1                 B_1( PORTJ, 2)
#define PJ3_1                 B_1( PORTJ, 3)
#define PJ4_1                 B_1( PORTJ, 4)
#define PJ5_1                 B_1( PORTJ, 5)
#define PJ6_1                 B_1( PORTJ, 6)
#define PJ7_1                 B_1( PORTJ, 7)
#define PJU_1                 PORTJ|= 0B11110000
#define PJL_1                 PORTJ|= 0B00001111
#define PJA_1                 PORTJ=  0B11111111

#define PJ0_0                 B_0( PORTJ, 0)
#define PJ1_0                 B_0( PORTJ, 1)
#define PJ2_0                 B_0( PORTJ, 2)
#define PJ3_0                 B_0( PORTJ, 3)
#define PJ4_0                 B_0( PORTJ, 4)
#define PJ5_0                 B_0( PORTJ, 5)
#define PJ6_0                 B_0( PORTJ, 6)
#define PJ7_0                 B_0( PORTJ, 7)
#define PJU_0                 PORTJ&= 0B00001111
#define PJL_0                 PORTJ&= 0B11110000
#define PJA_0                 PORTJ=  0B00000000

#define PJ0_RD                B_R( PINJ, 0)
#define PJ1_RD                B_R( PINJ, 1)
#define PJ2_RD                B_R( PINJ, 2)
#define PJ3_RD                B_R( PINJ, 3)
#define PJ4_RD                B_R( PINJ, 4)
#define PJ5_RD                B_R( PINJ, 5)
#define PJ6_RD                B_R( PINJ, 6)
#define PJ7_RD                B_R( PINJ, 7)
#define PJU_RD                (( PINJ & 0B11110000) >> 4)
#define PJL_RD                (PINJ & 0B00001111)
#define PJA_RD                PINJ

#define PJ0_WR( X)            X ? PJ0_1 : PJ0_0
#define PJ1_WR( X)            X ? PJ1_1 : PJ1_0
#define PJ2_WR( X)            X ? PJ2_1 : PJ2_0
#define PJ3_WR( X)            X ? PJ3_1 : PJ3_0
#define PJ4_WR( X)            X ? PJ4_1 : PJ4_0
#define PJ5_WR( X)            X ? PJ5_1 : PJ5_0
#define PJ6_WR( X)            X ? PJ6_1 : PJ6_0
#define PJ7_WR( X)            X ? PJ7_1 : PJ7_0
#define PJU_WR( X)            ( PORTJ& 0B00001111)| ( X<< 4)
#define PJL_WR( X)            ( PORTJ& 0B11110000)| ( X & 0B00001111)
#define PJA_WR( X)            PORTJ= X




#define K_PORT               0x0A00
#define PK0_PIN               J_PORT+ 0B00000001
#define PK1_PIN               K_PORT+ 0B00000010
#define PK2_PIN               K_PORT+ 0B00000100
#define PK3_PIN               K_PORT+ 0B00001000
#define PK4_PIN               K_PORT+ 0B00010000
#define PK5_PIN               K_PORT+ 0B00100000
#define PK6_PIN               K_PORT+ 0B01000000
#define PK7_PIN               K_PORT+ 0B10000000

#define PK0_OUTPUT            B_1( DDRK, 0)
#define PK1_OUTPUT            B_1( DDRK, 1)
#define PK2_OUTPUT            B_1( DDRK, 2)
#define PK3_OUTPUT            B_1( DDRK, 3)
#define PK4_OUTPUT            B_1( DDRK, 4)
#define PK5_OUTPUT            B_1( DDRK, 5)
#define PK6_OUTPUT            B_1( DDRK, 6)
#define PK7_OUTPUT            B_1( DDRK, 7)
#define PKU_OUTPUT            DDRK|= 0B11110000
#define PKL_OUTPUT            DDRK|= 0B00001111
#define PKA_OUTPUT            DDRK=  0B11111111

#define PK0_INPUT             B_0( DDRK, 0)
#define PK1_INPUT             B_0( DDRK, 1)
#define PK2_INPUT             B_0( DDRK, 2)
#define PK3_INPUT             B_0( DDRK, 3)
#define PK4_INPUT             B_0( DDRK, 4)
#define PK5_INPUT             B_0( DDRK, 5)
#define PK6_INPUT             B_0( DDRK, 6)
#define PK7_INPUT             B_0( DDRK, 7)
#define PKU_INPUT             DDRK&= 0B00001111
#define PKL_INPUT             DDRK&= 0B11110000
#define PKA_INPUT             DDRK=  0

#define PK0_PULLUP            PK0_INPUT; PK0_1
#define PK1_PULLUP            PK1_INPUT; PK1_1
#define PK2_PULLUP            PK2_INPUT; PK2_1
#define PK3_PULLUP            PK3_INPUT; PK3_1
#define PK4_PULLUP            PK4_INPUT; PK4_1
#define PK5_PULLUP            PK5_INPUT; PK5_1
#define PK6_PULLUP            PK6_INPUT; PK6_1
#define PK7_PULLUP            PK7_INPUT; PK7_1
#define PKU_PULLUP            PKU_INPUT; PKU_1
#define PKL_PULLUP            PKL_INPUT; PKL_1
#define PKA_PULLUP            PKA_INPUT; PKA_1

#define PK0_1                 B_1( PORTK, 0)
#define PK1_1                 B_1( PORTK, 1)
#define PK2_1                 B_1( PORTK, 2)
#define PK3_1                 B_1( PORTK, 3)
#define PK4_1                 B_1( PORTK, 4)
#define PK5_1                 B_1( PORTK, 5)
#define PK6_1                 B_1( PORTK, 6)
#define PK7_1                 B_1( PORTK, 7)
#define PKU_1                 PORTK|= 0B11110000
#define PKL_1                 PORTK|= 0B00001111
#define PKA_1                 PORTK=  0B11111111

#define PK0_0                 B_0( PORTK, 0)
#define PK1_0                 B_0( PORTK, 1)
#define PK2_0                 B_0( PORTK, 2)
#define PK3_0                 B_0( PORTK, 3)
#define PK4_0                 B_0( PORTK, 4)
#define PK5_0                 B_0( PORTK, 5)
#define PK6_0                 B_0( PORTK, 6)
#define PK7_0                 B_0( PORTK, 7)
#define PKU_0                 PORTK&= 0B00001111
#define PKL_0                 PORTK&= 0B11110000
#define PKA_0                 PORTK=  0B00000000

#define PK0_RD                B_R( PINK, 0)
#define PK1_RD                B_R( PINK, 1)
#define PK2_RD                B_R( PINK, 2)
#define PK3_RD                B_R( PINK, 3)
#define PK4_RD                B_R( PINK, 4)
#define PK5_RD                B_R( PINK, 5)
#define PK6_RD                B_R( PINK, 6)
#define PK7_RD                B_R( PINK, 7)
#define PKU_RD                (( PINK & 0B11110000) >> 4)
#define PKL_RD                (PINK & 0B00001111)
#define PKA_RD                PINK

#define PK0_WR( X)            X ? PK0_1 : PK0_0
#define PK1_WR( X)            X ? PK1_1 : PK1_0
#define PK2_WR( X)            X ? PK2_1 : PK2_0
#define PK3_WR( X)            X ? PK3_1 : PK3_0
#define PK4_WR( X)            X ? PK4_1 : PK4_0
#define PK5_WR( X)            X ? PK5_1 : PK5_0
#define PK6_WR( X)            X ? PK6_1 : PK6_0
#define PK7_WR( X)            X ? PK7_1 : PK7_0
#define PKU_WR( X)            ( PORTK& 0B00001111)| ( X<< 4)
#define PKL_WR( X)            ( PORTK& 0B11110000)| ( X & 0B00001111)
#define PKA_WR( X)            PORTK= X




#endif